/* eslint-disable no-unused-vars */
import io from "socket.io-client";
import {
    CookieStorage
} from "cookie-storage";
import striptags from "striptags";
import axios from "axios";

import log from "./log.js";
import math from "../utilities/math.js";

const cookie = new CookieStorage();

// http://localhost:3000
// http://localhost:2428/tanks
const APIUrl = "https://api.arnweb.nl/tanks";
const GameID = "ms99";

class Multiplayer {
    constructor(game) {
        // Keep local and external players 
        this.players = {};
        this.connected = false;
        this.game = game;

        this.lastPlaying;

        this.game.addListener("sendBomb", () => {
            this.sendBomb.call(this);
        });

        this.game.addListener("cleared", () => {
            this.cleared.call(this);
        });

        this.game.addListener("gameOver", () => {
            this.gameOver.call(this);
        });

        log("Game instance initialized");

        var username = cookie.getItem(`${GameID}_user`);
        var opponents = sessionStorage.getItem(`${GameID}_settings`);
        if (username != null && opponents != null) {
            log(`Using previous username ${username} and settings, close this session to reset this.`, "alert");
            this.connect(username, opponents);
        } else {
            this.register(username);
        }
    }

    connect(username, opponents) {
        // Show gamelog
        document.querySelectorAll("div#log")[0].style.display = "block";

        // Connect to TanksJS-Server instance
        log("Connecting to server...");
        this.socket = io(APIUrl, {
            reconnection: false
        });

        this.socket.on("connect_error", err => this.handleError(err));
        this.socket.on("connect_failed", err => this.handleError(err));
        this.socket.on("disconnect", err => this.handleError(err));

        this.socket.on("identify", () => {
            this.socket.emit("identification", {
                name: username,
                playersMax: opponents,
                gameID: GameID
            });
            log(`Connected to server as ${username}`, "success");
            this.connected = true;

            log(`Client ID: ${this.socket.id}`);
            console.log(`Client ID: ${this.socket.id}`);
        });

        // Handle player logins    
        this.socket.on("roomUpdate", (data) => {
            // Add any player that is not the local player and is not already added
            for (const id in data) {
                if (id != this.socket.id && this.players[id] == undefined) {
                    this.players[id] = data[id];
                    log(`${this.players[id].username} has joined`, "success");
                }
            }

            // Remove any player that disconnects from the game
            for (const id in this.players) {
                if (data[id] == undefined) {
                    log(`${this.players[id].username} has left.`, "alert");
                    delete this.players[id];
                    if (Object.keys(this.players).length == 0 && this.game.active) {
                        log("No other players left", "alert");
                        this.game.gameOver(true);
                    }
                }
            }

            this.opponents = opponents;

            this.game.drawStats({
                current: Object.keys(this.players).length,
                max: opponents
            });
        });

        // Handle player updates
        this.socket.on("update", (data) => {
            if (data.player.cleared) {
                this.game.gameOver();
            }

            if (data.player.target) {
                if (data.player.target.id == this.socket.id) {
                    this.game.addBomb();
                }
            } else {
                if (!data.player.active) {
                    log(`${this.players[data.uuid].username} has died`);

                    if (Object.keys(this.players).length == 1) {
                        this.lastPlaying = this.players[Object.keys(this.players)[0]];
                    }

                    delete this.players[data.uuid];

                    if (Object.keys(this.players).length == 0) {
                        if (this.game.active) {
                            log("No other players left", "alert");
                            this.game.gameOver(true);
                        } else {
                            log(`${this.lastPlaying.username} won`);
                        }
                    }
                }
            }

            this.game.drawStats({
                current: Object.keys(this.players).length,
                max: opponents
            });
        });

        // Start game
        this.socket.on("gameStart", () => {
            this.game.active = true;
            log("Room is full, starting game!", "alert");
        });
    }

    register(username = "") {
        var form = document.querySelectorAll("div#register > form")[0];
        form.parentElement.style.display = "block";

        axios.get(APIUrl).then((res) => {
            var output = document.getElementById("query");
            if (res.status == 200) {
                var result = res.data.players;

                var span = document.createElement("span");
                if (Object.keys(result).length == 0) {
                    span.style.color = "orange";
                    span.innerHTML = "No players are online :(";
                    output.appendChild(span);
                } else {
                    var rooms = [];
                    var counts = {};

                    for (var player in result) {
                        var room = result[player].room.split("_");
                        if (room[0] == GameID) {
                            rooms.push(room[1]);
                        }
                    }

                    for (var i = 0; i < rooms.length; i++) {
                        var num = rooms[i];
                        counts[num] = counts[num] ? counts[num] + 1 : 1;
                    }

                    for (const size in counts) {
                        span.innerHTML += String(`${counts[size]} player(s) in groups of ${size}.<br>`);
                    }

                    output.appendChild(span);
                }
            } else {
                span.style.color = "red";
                span.innerHTML = "Server is offline!";
                output.appendChild(span);
                form.querySelector("button").disabled = true;
            }
        });

        var checks = {
            session: form.querySelector("input[name=session]"),
            store: form.querySelector("input[name=store]"),
        };

        form.querySelector("input[name=username]").value = username;
        if (username != "") {
            checks.store.checked = true;
        }

        checks.session.addEventListener("change", function () {
            if (this.checked) {
                checks.store.checked = true;
            }
        });

        checks.store.addEventListener("change", function () {
            if (!this.checked) {
                checks.session.checked = false;
            }
        });

        form.addEventListener("submit", (e) => {
            e.preventDefault();
            form.parentElement.style.display = "none";

            var username = String(striptags(form.querySelector("input[name=username]").value));
            var opponents = form.querySelector("input[name=opponents]").value;

            if (form.querySelector("input[name=store]").checked) {
                cookie.setItem(`${GameID}_user`, username);
            }

            if (form.querySelector("input[name=session]").checked) {
                cookie.setItem(`${GameID}_user`, username);
                sessionStorage.setItem(`${GameID}_settings`, opponents);
            }

            this.connect(username, opponents);
        });
    }

    handleError(error) {
        log(error, "error");
    }

    sendBomb() {
        var rand = math.rand(0, Object.keys(this.players).length);

        this.socket.emit("update", {
            active: this.game.active,
            target: {
                id: Object.keys(this.players)[rand]
            }
        });
    }

    cleared() {
        this.socket.emit("update", {
            cleared: true
        });
    }

    gameOver() {
        this.socket.emit("update", {
            active: this.game.active
        });

        this.game.drawStats({
            current: Object.keys(this.players).length,
            max: this.opponents
        });

        var button = document.getElementById("restart");
        button.disabled = false;
        button.addEventListener("click", (e) => {
            this.socket.disconnect();
            window.location.reload();
        });
    }
}

export default Multiplayer;