import math from "../utilities/math.js";
import log from "../helpers/log.js";
import {
    EventEmitter
} from "events";

import bomb from "../../res/svg/bomb-solid.svg";
import flag from "../../res/svg/flag-solid.svg";

class Game extends EventEmitter {
    constructor(element) {
        super();

        this.board = element;
        this.children = element.children;
        this.colours = [
            "#B8E5E5",
            "#A9C6E5",
            "#A79AE6",
            "#CE8BE7",
            "#E97DC1",
            "#EB6D6E",
            "#ED9B5C",
            "#ECED4C"
        ];

        this.active = false;

        this.sides = 20;
        this.size = Math.pow(this.sides, 2);
        this.board.style.gridTemplateColumns = `repeat(${this.sides}, auto)`;

        this.generated = false;
        this.bombAmount = 50;
        this.bombs = [];
        this.added = [];
        this.discovered = [];

        this.time = 60;
        this.penalty = 20;

        for (let cell = 0; cell < this.size; cell++) {
            this.bombs.push(0);
            this.discovered.push(0);

            var child = document.createElement("span");
            child.className = "cell";

            child.addEventListener("click", () => {
                this.discover(cell);
            });

            child.addEventListener("contextmenu", (e) => {
                e.preventDefault();
                this.flag(cell);
            });

            this.board.appendChild(child);
        }
    }

    update() {
        if (this.active) {
            this.time--;
            if (this.time <= 0) {
                this.gameOver();
            } else {
                this.drawStats();
            }
        }
    }

    drawStats(players = undefined) {
        var minutes = 0;

        while (this.time - 60 - (minutes * 60) >= 0) {
            minutes++;
        }
        var seconds = this.time - (minutes * 60);

        if (minutes < 10) {
            minutes = String("0" + minutes);
        }

        if (seconds < 10) {
            seconds = String("0" + seconds);
            if (this.time < 0) {
                seconds = String("00");
            }
        }

        var timer = document.getElementById("stats").children[1];
        timer.innerHTML = String(minutes + ":" + seconds);

        var bombCounter = document.getElementById("bombs").children[1];
        bombCounter.innerHTML = String(this.bombAmount);

        if (players != undefined) {
            var playersStats = document.getElementById("players").children[1];
            if (this.active || !this.generated) {
                players.current++;
            }
            playersStats.innerHTML = String(players.current + "/" + players.max);
        }
    }

    surrounds(cell) {
        var surrounds = [];
        var limit = {
            top: true,
            bottom: true
        };

        if (!(cell - this.sides < 0)) {
            // Cell is not on top of the map
            limit.top = false;
            surrounds.push(
                cell - this.sides
            );
        }

        if (!(cell + this.sides > this.size - 1)) {
            // Cell is not on the bottom of the map
            limit.bottom = false;
            surrounds.push(
                cell + this.sides
            );
        }

        if (cell % this.sides === 0) {
            // Cell is on the left side of the map
            surrounds.push(cell + 1);
            if (!limit.top) {
                surrounds.push(cell - this.sides + 1);
            }
            if (!limit.bottom) {
                surrounds.push(cell + this.sides + 1);
            }
        } else if ((cell + 1) % this.sides === 0) {
            // Cell is on the right side of the map
            surrounds.push(cell - 1);
            if (!limit.top) {
                surrounds.push(cell - this.sides - 1);
            }
            if (!limit.bottom) {
                surrounds.push(cell + this.sides - 1);
            }
        } else {
            // Cell is not on either sides of the map
            surrounds.push(cell - 1);
            surrounds.push(cell + 1);
            if (!limit.top) {
                surrounds.push(cell - this.sides - 1);
                surrounds.push(cell - this.sides + 1);
            }
            if (!limit.bottom) {
                surrounds.push(cell + this.sides - 1);
                surrounds.push(cell + this.sides + 1);
            }
        }
        return surrounds;
    }

    discover(cell) {
        if (!this.generated) {
            this.generated = true;

            let index = 0;
            while (index < this.bombAmount) {
                var rand = math.rand(0, this.size);
                if (rand != cell && !this.bombs[rand] && this.surrounds(cell).indexOf(rand) == -1) {
                    this.bombs[rand] = 1;
                    index++;
                }
            }
        }

        if (this.active && !this.discovered[cell]) {
            if (this.bombs[cell]) {
                this.gameOver();
                return;
            }

            this.discovered[cell] = 1;
            this.children[cell].classList.add("discovered");

            var surrounds = this.surrounds(cell);

            var value = 0;
            surrounds.forEach((number) => {
                if (number != cell) {
                    if (this.bombs[number]) {
                        value++;
                    }
                }
            });

            if (value == 0) {
                surrounds.forEach((number) => {
                    if (number != cell) {
                        this.discover(number);
                    }
                });
            } else {
                this.children[cell].style.color = this.colours[value];
                this.children[cell].innerHTML = String(value);
            }
        }
    }

    flag(cell) {
        if (this.active) {
            if (this.bombs[cell]) {
                if (!this.discovered[cell]) {
                    this.discovered[cell] = 1;

                    var img = document.createElement("img");
                    img.src = flag;
                    this.children[cell].appendChild(img);
                    this.children[cell].classList.add("flag");

                    this.bombAmount--;

                    this.emit("sendBomb");

                    this.time = this.time + 10;
                    this.drawStats();

                    if (this.bombAmount == 0) {
                        log("You have cleared all bombs!", "success");
                        this.emit("cleared");
                    }
                }
            } else {
                if (!this.discovered[cell]) {
                    this.time = this.time - this.penalty;
                    this.drawStats();

                    this.discover(cell);
                }
            }
        }
    }

    addBomb() {
        for (let tries = 0; tries < 10; tries++) {
            var sum = 0;

            var cell = math.rand(0, this.bombs.length);

            // var sum = this.surrounds(cell).reduce((a, b) => { 
            //     console.log(this.discovered[a]);
            //     return this.discovered[a] + this.discovered[b];
            // }, 0);

            this.surrounds(cell).forEach((e) => {
                sum = sum + this.discovered[e];
            });

            if (sum == 0 && !this.bombs[cell] && !this.discovered[cell]) {
                this.bombs[cell] = 1;
                this.bombAmount++;
                this.added.push(cell);

                // this.children[cell].style.background = "#0000ff";
                log("A bomb has been added to your field!", "alert");
                return;
            }
        }
    }

    gameOver(win = false) {
        log("Game over", "error");
        if (win) {
            log("Congratulations, you win!", "success");
        }

        this.active = false;
        this.drawStats();

        this.emit("gameOver");

        for (let index = 0; index < this.size; index++) {
            if (this.bombs[index]) {
                if (!this.discovered[index]) {
                    var img = document.createElement("img");
                    img.src = bomb;
                    this.children[index].appendChild(img);
                    this.children[index].classList.add("bomb");
                    if (this.added.indexOf(index) != -1) {
                        this.children[index].classList.add("added");
                    }
                }
            }
        }
    }
}

export default Game;